MODULE zdfbfr
   !!======================================================================
   !!                       ***  MODULE  zdfbfr  ***
   !! Ocean physics: Bottom friction
   !!======================================================================
   !! History :  OPA  ! 1997-06  (G. Madec, A.-M. Treguier)  Original code
   !!   NEMO     1.0  ! 2002-06  (G. Madec)  F90: Free form and module
   !!            3.2  ! 2001-09  (A.C.Coward)  Correction to include barotropic contribution
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   zdf_bfr      : update momentum Kz at the ocean bottom due to the type of bottom friction chosen
   !!   zdf_bfr_init : read in namelist and control the bottom friction parameters.
   !!   zdf_bfr_2d   : read in namelist and control the bottom friction
   !!                  parameters.
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers variables
   USE dom_oce         ! ocean space and time domain variables 
   USE zdf_oce         ! ocean vertical physics variables
   USE in_out_manager  ! I/O manager
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE lib_mpp         ! distributed memory computing
   USE prtctl          ! Print control

   IMPLICIT NONE
   PRIVATE

   PUBLIC   zdf_bfr    ! called by step.F90

   !                                   !!* Namelist nambfr: bottom friction namelist *
   INTEGER  ::   nn_bfr   = 0           ! = 0/1/2/3 type of bottom friction 
   REAL(wp) ::   rn_bfri1 = 4.0e-4_wp   ! bottom drag coefficient (linear case) 
   REAL(wp) ::   rn_bfri2 = 1.0e-3_wp   ! bottom drag coefficient (non linear case)
   REAL(wp) ::   rn_bfeb2 = 2.5e-3_wp   ! background bottom turbulent kinetic energy  [m2/s2]
   REAL(wp) ::   rn_bfrien = 30_wp      ! local factor to enhance coefficient bfri
   REAL(wp), DIMENSION(jpi,jpj) :: &
                 bfrcoef2d = 1.e-3_wp   ! 2D bottom drag coefficient
   LOGICAL  ::   ln_bfr2d = .false.     ! logical switch for 2D enhancement
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) :: & 
      &          bfrua , bfrva          ! Bottom friction coefficients set in zdfbfr

   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.2 , LOCEAN-IPSL (2009) 
   !! $Id$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE zdf_bfr( kt )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE zdf_bfr  ***
      !!                 
      !! ** Purpose :   Applied the bottom friction through a specification of 
      !!              Kz at the ocean bottom.
      !!
      !! ** Method  :   Calculate and store part of the momentum trend due    
      !!              to bottom friction following the chosen friction type 
      !!              (free-slip, linear, or quadratic). The component
      !!              calculated here is multiplied by the bottom velocity in
      !!              dyn_bfr to provide the trend term.
      !!
      !! ** Action  :   bfrua , bfrva   bottom friction coefficients
      !!----------------------------------------------------------------------
      INTEGER, INTENT( in ) ::   kt   ! ocean time-step index
      !!
      INTEGER  ::   ji, jj         ! dummy loop indexes
      INTEGER  ::   ikbu, ikbum1   ! temporary integers
      INTEGER  ::   ikbv, ikbvm1   ! temporary integers
      REAL(wp) ::   zvu, zuv, zecu, zecv   ! temporary scalars
      !!----------------------------------------------------------------------


      IF( kt == nit000 )   CALL zdf_bfr_init   ! initialisation

      IF( nn_bfr == 2 ) THEN                 ! quadratic botton friction
         ! Calculate and store the quadratic bottom friction terms bfrua and bfrva
         ! where bfrUa = C_d*SQRT(u_bot^2 + v_bot^2 + e_b) {U=[u,v]}
         ! from these the trend due to bottom friction:  -F_h/e3U  can be calculated
         ! where -F_h/e3U_bot = bfrUa*Ub/e3U_bot {U=[u,v]}
         !
# if defined key_vectopt_loop
         DO jj = 1, 1
!CDIR NOVERRCHK
            DO ji = jpi+2, jpij-jpi-1   ! vector opt. (forced unrolling)
# else
!CDIR NOVERRCHK
         DO jj = 2, jpjm1
!CDIR NOVERRCHK
            DO ji = 2, jpim1
# endif
               ikbu   = MIN( mbathy(ji+1,jj  ), mbathy(ji,jj) )
               ikbv   = MIN( mbathy(ji  ,jj+1), mbathy(ji,jj) )
               ikbum1 = MAX( ikbu-1, 1 )
               ikbvm1 = MAX( ikbv-1, 1 )
               !
               zvu  = 0.25 * (  vn(ji,jj  ,ikbum1) + vn(ji+1,jj  ,ikbum1)     &
                  &           + vn(ji,jj-1,ikbum1) + vn(ji+1,jj-1,ikbum1)  )
               zuv  = 0.25 * (  un(ji,jj  ,ikbvm1) + un(ji-1,jj  ,ikbvm1)     &
                  &           + un(ji,jj+1,ikbvm1) + un(ji-1,jj+1,ikbvm1)  )
               !
               zecu = SQRT(  un(ji,jj,ikbum1) * un(ji,jj,ikbum1) + zvu*zvu + rn_bfeb2  )
               zecv = SQRT(  vn(ji,jj,ikbvm1) * vn(ji,jj,ikbvm1) + zuv*zuv + rn_bfeb2  )
               !
               bfrua(ji,jj) = - bfrcoef2d(ji,jj) * zecu 
               bfrva(ji,jj) = - bfrcoef2d(ji,jj) * zecv
            END DO
         END DO
         !
      ENDIF
      !
      CALL lbc_lnk( bfrua, 'U', 1. )   ;   CALL lbc_lnk( bfrva, 'V', 1. )      ! Lateral boundary condition

      IF(ln_ctl)   CALL prt_ctl( tab2d_1=bfrua, clinfo1=' bfr  - u: ', mask1=umask,        &
         &                       tab2d_2=bfrva, clinfo2=       ' v: ', mask2=vmask,ovlap=1 )
      !
   END SUBROUTINE zdf_bfr


   SUBROUTINE zdf_bfr_init
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE zdf_bfr_init  ***
      !!                    
      !! ** Purpose :   Initialization of the bottom friction
      !!
      !! ** Method  :   Read the nammbf namelist and check their consistency
      !!              called at the first timestep (nit000)
      !!----------------------------------------------------------------------
      USE iom   ! I/O module for ehanced bottom friction file
      !!
      NAMELIST/nambfr/ nn_bfr, rn_bfri1, rn_bfri2, rn_bfeb2, ln_bfr2d, rn_bfrien
      INTEGER ::   inum       ! logical unit for enhanced bottom friction file
      INTEGER ::   ji, jj     ! dummy loop indexes
      INTEGER ::   ikbu, ikbv ! temporary integers
      INTEGER ::   ictu, ictv ! temporary integers
      REAL(wp) ::  rminbfr, rmaxbfr
                              ! temporary reals
      !!----------------------------------------------------------------------

      REWIND ( numnam )               !* Read Namelist nam_bfr : bottom momentum boundary condition
      READ   ( numnam, nambfr )


      !                               !* Parameter control and print
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'zdf_bfr : momentum bottom friction'
      IF(lwp) WRITE(numout,*) '~~~~~~~'
      IF(lwp) WRITE(numout,*) '          Namelist nam_bfr : set bottom friction parameters'

      SELECT CASE (nn_bfr)

      CASE( 0 )
         IF(lwp) WRITE(numout,*) '            free-slip '
         !
         bfrua(:,:) = 0.e0
         bfrva(:,:) = 0.e0
         !
      CASE( 1 )
         IF(lwp) WRITE(numout,*) '            linear botton friction'
         IF(lwp) WRITE(numout,*) '            friction coef.   rn_bfri1  = ', rn_bfri1
         IF(lwp)  THEN
            IF( ln_bfr2d ) THEN
                 WRITE(numout,*) '            read coef. enhancement distribution from file   ln_bfr2d  = ', ln_bfr2d
                 WRITE(numout,*) '            coef rn_bfri2 enhancement factor   rn_bfrien  = ',rn_bfrien
            ENDIF
         ENDIF
         !
         bfrcoef2d(:,:) = rn_bfri1  ! initialize bfrcoef2d to the namelist variable

         IF (ln_bfr2d) THEN 
            ! bfr_coef is a coefficient in [0,1] giving the mask where to apply the bfr enhancement
            CALL iom_open('bfr_coef.nc',inum)
            CALL iom_get (inum, jpdom_data, 'bfr_coef',bfrcoef2d,1) ! bfrcoef2d is used as tmp array
            CALL iom_close(inum)
            bfrcoef2d(:,:)= rn_bfri1*(1+ rn_bfrien*bfrcoef2d(:,:))
         ENDIF
         bfrua(:,:) = - bfrcoef2d(:,:)
         bfrva(:,:) = - bfrcoef2d(:,:)
         !
      CASE( 2 )
         IF(lwp) WRITE(numout,*) '            quadratic botton friction'
         IF(lwp) WRITE(numout,*) '            friction coef.   rn_bfri2  = ', rn_bfri2
         IF(lwp) WRITE(numout,*) '            background tke   rn_bfeb2  = ', rn_bfeb2
         IF(lwp)  THEN
            IF( ln_bfr2d ) THEN
                 WRITE(numout,*) '            read coef. enhancement distribution from file   ln_bfr2d  = ', ln_bfr2d
                 WRITE(numout,*) '            coef rn_bfri2 enhancement factor   rn_bfrien  = ',rn_bfrien
            ENDIF
         ENDIF
         bfrcoef2d(:,:) = rn_bfri2  ! initialize bfrcoef2d to the namelist variable

         IF (ln_bfr2d) THEN 
            ! bfr_coef is a coefficient in [0,1] giving the mask where to apply the bfr enhancement
            CALL iom_open('bfr_coef.nc',inum)
            CALL iom_get (inum, jpdom_data, 'bfr_coef',bfrcoef2d,1) ! bfrcoef2d is used as tmp array
            CALL iom_close(inum)
            bfrcoef2d(:,:)= rn_bfri2*(1+ rn_bfrien*bfrcoef2d(:,:))
         ENDIF

         !
      CASE DEFAULT
         WRITE(ctmp1,*) '         bad flag value for nn_bfr = ', nn_bfr
         CALL ctl_stop( ctmp1 )
         !
      END SELECT
      !
      ! Basic stability check on bottom friction coefficient
      !
      ictu = 0               ! counter for stability criterion breaches at U-pts
      ictv = 0               ! counter for stability criterion breaches at V-pts
      rminbfr =  1.e10_wp    ! initialise tracker for minimum of bottom friction coefficient
      rmaxbfr = -1.e10_wp    ! initialise tracker for maximum of bottom friction coefficient
      !
#  if defined key_vectopt_loop
      DO jj = 1, 1
!CDIR NOVERRCHK
         DO ji = jpi+2, jpij-jpi-1   ! vector opt. (forced unrolling)
#  else
!CDIR NOVERRCHK
      DO jj = 2, jpjm1
!CDIR NOVERRCHK
         DO ji = 2, jpim1
#  endif
             ikbu   = MIN( mbathy(ji+1,jj  ), mbathy(ji,jj) )
             ikbv   = MIN( mbathy(ji  ,jj+1), mbathy(ji,jj) )
             IF ( bfrcoef2d(ji,jj).gt.2.0*fse3u(ji,jj,ikbu)/rdt ) then
                IF ( ln_ctl ) THEN
                   write(numout,*) 'BFR ',narea,nimpp+ji,njmpp+jj,ikbu
                   write(numout,*) 'BFR ',bfrcoef2d(ji,jj),2.0*fse3u(ji,jj,ikbu)/rdt
                ENDIF
                ictu = ictu + 1
             ENDIF
             IF ( bfrcoef2d(ji,jj).gt.2.0*fse3v(ji,jj,ikbv)/rdt ) then
                 IF ( ln_ctl ) THEN
                     write(numout,*) 'BFR ',narea,nimpp+ji,njmpp+jj,ikbv
                     write(numout,*) 'BFR ',bfrcoef2d(ji,jj),2.0*fse3v(ji,jj,ikbv)/rdt
                 ENDIF
                 ictv = ictv + 1
             ENDIF
             bfrcoef2d(ji,jj) = MIN(2.0*fse3u(ji,jj,ikbu)/rdt, bfrcoef2d(ji,jj))
             bfrcoef2d(ji,jj) = MIN(2.0*fse3v(ji,jj,ikbv)/rdt, bfrcoef2d(ji,jj))
             rminbfr = MIN(rminbfr, bfrcoef2d(ji,jj))
             rmaxbfr = MAX(rmaxbfr, bfrcoef2d(ji,jj))
         END DO
      END DO
      IF ( lk_mpp ) THEN
         CALL mpp_sum( ictu )
         CALL mpp_sum( ictv )
         CALL mpp_min( rminbfr )
         CALL mpp_max( rmaxbfr )
      ENDIF
      IF ( lwp .AND. ictu + ictv .GT. 0 ) THEN
         WRITE(numout,*) ' Bottom friction stability check failed at ', ictu, ' U-points '
         WRITE(numout,*) ' Bottom friction stability check failed at ', ictv, ' V-points '
         WRITE(numout,*) ' Bottom friction coefficient reset where necessary'
         WRITE(numout,*) ' Bottom friction coefficient now ranges from: ', rminbfr, ' to ', rmaxbfr
      ENDIF
      !
   END SUBROUTINE zdf_bfr_init

   !!======================================================================
END MODULE zdfbfr
